@@ -716,7 +716,9 @@ public:
     char buf1[22],buf2[22];
 #endif
     DBUG_ENTER("harvest_bytes_written");
-    (*counter)+=bytes_written;
+
+    my_atomic_add64_explicit((volatile int64*)(counter), bytes_written,
+                             MY_MEMORY_ORDER_RELAXED);
     DBUG_PRINT("info",("counter: %s  bytes_written: %s", llstr(*counter,buf1),
 		       llstr(bytes_written,buf2)));
     bytes_written=0;
